% Gianni Schena  March 2007, schena@units.it
% Lattice Boltzmann LBE, geometry: D2Q9, model: BGK
% 2D channel with Poiseuille flow through ( North to South )
% 2 immiscible fluids R-K model (Rothman - Keller)

close all, clear all % start from scratch and clean ...
tic

%   IN
% |vvvv|    - y
% |vvvv|     |
% |vvvv|     | -> + x
%  OUT       v +y
% FLOW
% Laminar parabolic flow in a 2d channel

% AVI File Mame to be generated 
avi_fname='IBt4_sigma_bassa'; % Immiscible Lattice Boltzmann Test 


Len_Channel_2D=128; % Length
Channel_2D_half_Width=16; % Half_Width
Width=Channel_2D_half_Width*2+1; % add 1 for symmetry 
Channel2D=ones(Len_Channel_2D,Width); % define wet area
[Nr Mc]=size(Channel2D); % Number rows and Munber columns
uy0=0; ux0=0; %  linear vel .. inizialization

uy_fin_max=0 ; % max Poiseuille final velocity on the flow profile
uyf_av=uy_fin_max*(2/3); % average fluid velocity on the profile
Char_Dim = Width; % characteristic length-scale of the duct or obstacle
Rey_N = 20; % Reynolds number
Lky_visco = uy_fin_max * Char_Dim / Rey_N; % kinematic viscosity
omega = 1. / (3*Lky_visco+1./2.); % relaxation parameter
cs2=1/3; 
Lky_visco=cs2*(1/omega - 0.5) ; % recal. lattice kinematic viscosity

omega=1; % omega: relaxation frequency


dPdL = ( 2*Lky_visco*uy_fin_max/(Channel_2D_half_Width.^2) ); % Poiseuille Gradient;

Poiseuille_Profile; % uy_analy_profile= ... 

PixelSize= 5; % [Microns]
dL=(Nr*PixelSize*1.0E-4); % sample hight [cm]

% COLOUR REFILL METHOD ALTERNAVIVES 

%Refill_method='Latvo_Kakko'  ; % see M.Latvo kakko and Rothmann papers 
Refill_method='Keller'  ; % see Keller and Rothmann paper 


fct_Red=0.39; % volumetric fraction of Red fluid
Sigma=0.00003; % surface tension parameter
Ades_R_B=-1; 

%Capillary number ratio viscous forces to surface tension 
Capi_N= (uy_fin_max*Lky_visco)/Sigma; % says LT 10.^- 5 dominated by capillary forces.

%Bond number, dimensionless, ratio of body forces to surface tension forces:
Bond_N=dPdL/Sigma;

Red_and_Blue_Fluids; % properties

% EXPERIMENTAL SET-UP
Vessel = 'Channel2D'; % N to S laminar flow
%Vessel = 'Box2D'; % for bubble test
Vessel = 'doublet_of_pores'; % for cylindrical vessel / tank 

ILB_vessels; % include file

figure(10) % used to visualize evolution of rho
figure(11) % used to visualize ux
figure(12) % used to visualize uy (i.e. top -> down)

Compute_Absolute_Indices_for_Wet_Locations; 

D2Q9_Lattice; % Lattice & related matter 

Red_and_Blue_Fluids; 

Initialize_Velocities; % vectors linear vel .. inizialization


% INITIAL COLOUR DISTRIBUTION INTO WET LOCATIONS
% initialization arrays : start Kolour values in the wet area
%Initial_Colour_Distribution = 'Rand_Unif'
%Initial_Colour_Distribution ='Vert_Interf';
%Initial_Colour_Distribution ='Horiz_Interf';
Initial_Colour_Distribution= 'horiz_interf_flip';
%Initial_Colour_Distribution ='Red_Lane';
%Initial_Colour_Distribution ='Red_Buble2D';
%Initial_Colour_Distribution = 'red_stripe';

ILB_initial_RB_colours; % include file 

Define_Body_Force;    
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % distributions
        % Blue or Red !
        f=Red+Blue; % f is 0 in the dry areas
        rhoR=sum(Red,3); rhoB=sum(Blue,3); % colour density
        rho=rhoR+rhoB; %

        grap_out_level=1
        
        Avi_Initialization2; 
        
        uy(ija)=uy0; ux(ija)=ux0; % initialize fluid velocities

        % While .. MAIN TIME EVOLUTION LOOP
        av_vel_t=1.e+10; % inizialization (t=0)
        StopFlag=false; % i.e. logical(0)
        Max_Iter=280000; % max allowed number of iteration
        Check_Iter=1000; Output_Every=1000; % frequency of check & output
        Cur_Iter=0; % current iteration counter inizialization
        toler=1.0e-7; % tollerance to declare convegence
        Cond_path=[]; % recording values of the convergence criterium
        density_path=[]; % recording aver. density values for convergence
        
         %%matlabpool close 
     %   matlabpool open local 4, 
        % call main time evolution loop
        
        Main_Time_Evolution_Loop
        
        % Output & Draw after the end of the time evolution

        figure, plot(Cond_path(2:end)); title('convergence path')
        %figure, plot(density_path(2:end)); title('density convergence path')
        %figure, plot( [uy(Nr-up,:)-uy_analy_profile] ); title('LB - Analytical solution')

        toc %
     Close_Avi
       